//      

                                 
                                                                           

const { v4: uuid } = require('uuid');
const Debouncer = require('./Debouncer');

class TagSet                            {
  /**
   * The store.
   *
   * @var {Redis}
   */
               

  /**
   * The tag names.
   *
   * @var array
   */
                       

  /**
   * The namespace.
   *
   * @var string
   */
                    

  /**
   * The debouncer.
   *
   * @var {DebouncerInterface}
   */
                                

  /**
   * @inheritDoc
   */
  constructor(store       , names               ) {
    this.store = store;
    this.names = names;
    this.debouncer = new Debouncer(store);
  }

  /**
   * @inheritDoc
   */
  reset()                {
    // Generate new IDs for all the names and set them all at the same time.
    return Promise.all(this.getNames()
      .map(name => [
        this.tagKey(name),
        uuid().replace(/-/g, ''),
      ])
      .map(tuple => this.store.set(...tuple)))
      .then(() => {});
  }

  /**
   * @inheritDoc
   */
  initTag(name        )                  {
    const id = uuid().replace(/-/g, '');
    return this.store.setnx(name, id)
      .then(res => res ? id : this.debouncer.debounce('get', name));
  }

  /**
   * @inheritDoc
   */
  getNamespace()                  {
    if (this.namespace) {
      return Promise.resolve(this.namespace);
    }
    return this.tagIds().then(ids => {
      this.namespace = ids.join('|');
      return this.namespace;
    });
  }

  /**
   * @inheritDoc
   */
  tagIds()                         {
    const names = this.getNames();
    const tagKeys = names.map(name => this.tagKey(name));
    return Promise.all(tagKeys.map(k => this.debouncer.debounce('get', k)))
      .then(tags => {
        // If there is a tag associated to the name, get it. If not, create it.
        const fillTags = (tag        , index        ) => tag
          ? Promise.resolve(tag)
          : this.initTag(tagKeys[index]);
        const promises = tags.map((t, index) => fillTags(t, index));
        return Promise.all(promises);
      });
  }

  /**
   * @inheritDoc
   */
  tagId(name        )                  {
    const key = this.tagKey(name);
    return this.debouncer.debounce('get', key) || this.initTag(key);
  }

  /**
   * @inheritDoc
   */
  tagKey(name        )         {
    return `tag:${name}:key`;
  }

  /**
   * @inheritDoc
   */
  getNames()                {
    return this.names;
  }
}

module.exports = TagSet;
